
#include <xc.h>
#include "i2c_emem.h"
#include "RTCC_MCP79410.h"

void                            Delay_for_EEW(void);

extern uint16_t        wBuff[10], rBuff[10];
I2CEMEM_DRV         i2cmem = I2CSEMEM_DRV_DEFAULTS ;
I2CEMEM_DATA      wData;
I2CEMEM_DATA      rData;

extern unsigned char   Station_TimeLog[32][6] ;

Time_BUF    Time_Buf ; 

uint16_t        wBuff[10], rBuff[10];

union   TIME_RTCSEC
{
    struct  {
        unsigned SECONE : 4 ;
       unsigned SECTEN : 3 ;
        unsigned ST : 1 ;      
    };
    unsigned char   SECOND ;
}RTCC_Second;

union   TIME_RTCMIN
{
    struct  {
        unsigned MINONE : 4 ;
        unsigned MINTEN : 3 ;
        unsigned : 1 ;
        
        
    };
    unsigned char   MINUTE ;
}RTCC_Minute;

union   TIME_RTCHOUR
{
    struct  {
        unsigned HRONE : 4 ;
        unsigned HRTEN : 2 ;
        unsigned TIME_FORMAT : 1 ;
        unsigned : 1 ;
    };
    struct{
        unsigned :4 ;
        unsigned HRTEN0 : 1 ;
        unsigned HRTEN1 : 1 ;
        unsigned : 2 ;
    };
    unsigned char   HOUR ;
}RTCC_Hour;


void    RTCC_Set_Time (int Hr, int Min, int Sec)
{
    RTCC_Hour.HRONE = Hr % 10 ;
    RTCC_Hour.HRTEN = Hr / 10 ;
    RTCC_Hour.TIME_FORMAT = 0 ;     // Select 24 Hours format 
    RTCC_Minute.MINONE = Min % 10 ;
    RTCC_Minute.MINTEN = Min / 10 ;
    RTCC_Second.SECONE = Sec % 10 ;
    RTCC_Second.SECTEN = Sec / 10 ;
    RTCC_Second.ST = 1 ;
    
                wBuff[0] = RTCC_Second.SECOND ;
                wBuff[1] = RTCC_Minute.MINUTE ;
                wBuff[2] = RTCC_Hour.HOUR ;
                wBuff[3] = 0x08 ;
               
                wData.buff = wBuff;
                wData.n = 4;                        // Write 3 bytes to RTCC to setup the time ...;
                wData.addr = 0x00;
                wData.csel = 0x00;    
                wData.Device_Addr = RTCC_ADDRESS ;
                
            i2cmem.oData = &wData;
            i2cmem.cmd = I2C_WRITE;     /* comment this line to omit the write phase */

            while( i2cmem.cmd != I2C_IDLE )
            {
                i2cmem.tick( &i2cmem );
            }
      
}

void    Delay_for_EEW(void)
{
    volatile unsigned long    M_Loop1 ;
    volatile unsigned long    M_Loop2 ;
    
    for (M_Loop2 = 0 ; M_Loop2 < 10; M_Loop2 ++)
        for (M_Loop1 = 0 ; M_Loop1 < 2000 ; M_Loop1 ++ ) ;
}


void        EEPROM_Write( int EE_Address, int   Byte_Count , unsigned int *WBuffer )
{
    int EE_W_Loop ;
    
    for (EE_W_Loop = 0 ; EE_W_Loop < Byte_Count; EE_W_Loop++ )
    {
            wBuff[EE_W_Loop] = *WBuffer ++ ;        // Move the desired data 
    }
                wData.buff = wBuff;
                wData.n = Byte_Count;                        // Write single byte to EEPROM
                wData.addr = EE_Address;
                wData.csel = 0x00;    
                wData.Device_Addr = EEPROM_ADDRESS ;  
                
            i2cmem.oData = &wData;
            i2cmem.cmd = I2C_WRITE;     /* comment this line to omit the write phase */

            while( i2cmem.cmd != I2C_IDLE )
            {
                i2cmem.tick( &i2cmem );
            }
            
             Delay_for_EEW( ) ;         
}

uint16_t    EEPROM_Read ( uint16_t   EE_Address)
{
          rData.buff = rBuff;
         rData.n = 1;                        //10;
         rData.addr = EE_Address;
         rData.csel = 0x00;  
         rData.Device_Addr = EEPROM_ADDRESS ;   
         
             i2cmem.oData = &rData;
            i2cmem.cmd = I2C_READ;      /* comment this line to omit the read phase */

            while( i2cmem.cmd != I2C_IDLE )
            {
                i2cmem.tick( &i2cmem );
            }
            //  Data already @ rData[0] - Time for Second Information
            return  rBuff[0] ;          
}

void    RTCC_Update (void)
{
 
      // Initialise I2C Data Object for Read operation
         rData.buff = rBuff;
         rData.n = 3;                        //10;
         rData.addr = 0x00;
         rData.csel = 0x00;  
         rData.Device_Addr = RTCC_ADDRESS ;

            // Read Data
            i2cmem.oData = &rData;
            i2cmem.cmd = I2C_READ;      /* comment this line to omit the read phase */

            while( i2cmem.cmd != I2C_IDLE )
            {
                i2cmem.tick( &i2cmem );
            }
            //  Data already @ rData[0] - Time for Second Information
            Time_Buf.Hour = rBuff[2] ;
            Time_Buf.Minute = rBuff[1] ;
            Time_Buf.Second = rBuff[0] ;
            
            Station_TimeLog[0][0] = rBuff[0] & 0b01111111 ;
            Station_TimeLog[0][1] = rBuff[1] & 0b01111111 ;
            Station_TimeLog[0][2] = rBuff[2] & 0b00011111 ;
            
}
