/*******************************************************************************
  ce427 main source file

  Company:
    Microchip Technology Inc.

  File Name:
    main.c

  Summary:
    This file consists of functions that send and receive messages through the
    ECAN1 and ECAN2 when the transmitter and receiver are crosswired.

*******************************************************************************/
/*******************************************************************************
Copyright (c) 2012 released Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.
*******************************************************************************/
// *****************************************************************************
// *****************************************************************************
// Section: Included Files
// *****************************************************************************
// *****************************************************************************
#include <xc.h>
#include "ecan1_config.h"
#include "common.h"
#include <stdint.h>
#include "i2c_emem.h"
#include "RTCC_MCP79410.h"
#include "E_Judger_LCM.h"
#include "timers.h"

#include "../Include_Files/lib_usb_cdc_mu810.h"
#include "../Include_Files/MP_Car_leds.h" 

#include <stdio.h>
#include <string.h>

// *****************************************************************************
// *****************************************************************************
// Section: Macros for Configuration Fuse Registers:
// *****************************************************************************
// *****************************************************************************
/* Invoke macros to set up  device configuration fuse registers.The fuses will
   select the oscillator source, power-up timers, watch-dog timers etc. The
   macros are defined within the device header files. The configuration fuse
   registers reside in Flash memory.
 */
// DSPIC33EP512MU810 Configuration Bit Settings
// 'C' source line config statements
// FGS
#pragma config GWRP = OFF           // General Segment Write-Protect bit (General Segment may be written)
#pragma config GSS = OFF              // General Segment Code-Protect bit (General Segment Code protect is disabled)
#pragma config GSSK = OFF           // General Segment Key bits (General Segment Write Protection and Code Protection is Disabled)

// FOSCSEL
//#pragma config FNOSC = PRIPLL         // Initial Oscillator Source Selection Bits (Internal Fast RC (FRC))
#pragma config FNOSC = PRIPLL
#pragma config IESO = ON           // Two-speed Oscillator Start-up Enable bit (Start up with user-selected oscillator source)

// FOSC
//#pragma config POSCMD = XT          // Primary Oscillator Mode Select bits (XT Crystal Oscillator Mode)
#pragma config POSCMD = HS
#pragma config OSCIOFNC = OFF       // OSC2 Pin Function bit (OSC2 is clock output)
#pragma config IOL1WAY = OFF        // Peripheral pin select configuration (Allow multiple reconfigurations)
#pragma config FCKSM = CSECMD       // Clock Switching Mode bits (Clock switching is enabled,Fail-safe Clock Monitor is disabled)

// FWDT
#pragma config WDTPOST = PS32768    // Watchdog Timer Postscaler Bits (1:32,768)
#pragma config WDTPRE = PR128       // Watchdog Timer Prescaler bit (1:128)
#pragma config PLLKEN = ON          // PLL Lock Wait Enable bit (Clock switch to PLL source will wait until the PLL lock signal is valid.)
#pragma config WINDIS = OFF         // Watchdog Timer Window Enable bit (Watchdog Timer in Non-Window mode)
#pragma config FWDTEN = OFF         // Watchdog Timer Enable bit (Watchdog timer enabled/disabled by user software)

// FPOR
#pragma config FPWRT = PWR128       // Power-on Reset Timer Value Select bits (128ms)
#pragma config BOREN = ON           // Brown-out Reset (BOR) Detection Enable bit (BOR is enabled)
#pragma config ALTI2C1 = ON       // Alternate I2C pins for I2C1 (SDA1/SCK1 pins are selected as the I/O pins for I2C1)
#pragma config ALTI2C2 = OFF        // Alternate I2C pins for I2C2 (SDA2/SCK2 pins are selected as the I/O pins for I2C2)

// FICD
#pragma config ICS = PGD1           // ICD Communication Channel Select bits (Communicate on PGEC1 and PGED1)
#pragma config RSTPRI = PF          // Reset Target Vector Select bit (Device will obtain reset instruction from Primary flash)
#pragma config JTAGEN = OFF         // JTAG Enable bit (JTAG is disabled)

// FAS
#pragma config AWRP = OFF           // Auxiliary Segment Write-protect bit (Auxiliary program memory is not write-protected)
#pragma config APL = OFF            // Auxiliary Segment Code-protect bit (Aux Flash Code protect is disabled)
#pragma config APLK = OFF           // Auxiliary Segment Key bits (Aux Flash Write Protection and Code Protection is Disabled)

// *****************************************************************************
// *****************************************************************************
// Section: File Scope or Global Constants
// *****************************************************************************
// *****************************************************************************
// Instantiate Drive and Data objects for I2C1


#define LED1_ON_TIME    50
#define LED2_ON_TIME    50
#define DEF_REMOTE_ID   0x200

union 
{
    struct  {
        unsigned DSW1 : 1 ;
        unsigned DSW2 : 1 ;
        unsigned DSW3 : 1 ;
        unsigned DSW4 : 1 ;
        unsigned DSW5 : 1 ;
        unsigned DSW6 : 1 ;
       unsigned : 2 ;
    };
    unsigned char   ID_Byte ;
}BOARD_ID;

union 
{
    struct  {
        unsigned TX0_Busy : 1 ;
        unsigned TX1_Busy : 1 ;
        unsigned TX2_Busy : 1 ;
        unsigned TX3_Busy : 1 ;
        unsigned TX4_Busy : 1 ;
        unsigned TX5_Busy : 1 ;
        unsigned TX6_Busy : 1 ;
        unsigned TX7_Busy : 1 ;
    };
    unsigned char  ALL_Busy  ;
}CANTX_FLAG;

 struct
{
    unsigned char   Hour ;
    unsigned char   Minute ;
    unsigned char   Second ;
} CLOCK1;

struct
{
    unsigned char   Hour ;
    unsigned char   Minute ;
    unsigned char   Second ;
} EXT_CLOCK1;

union
{
    struct
    {
        unsigned int    SID ;
        unsigned long   EID ;
        unsigned char   DLC ;
        unsigned char   IDE_BIT ;
        unsigned char   DATA_IN ;
        unsigned char   DATA_IN1 ;
        unsigned int    DataWord0 ;
        unsigned int    DataWord1 ;
        unsigned int    DataWord2 ;
        unsigned int    DataWord3 ;
    };
    struct
    {
        unsigned int    Null1 ;
        unsigned long   Null2 ;
        unsigned char   Null3 ;
        unsigned char   Null4 ;
        unsigned char   Null5 ;
        unsigned char   Null6 ;
        unsigned char   Byte0 ;
        unsigned char   Byte1 ;
        unsigned char   Byte2 ;
        unsigned char   Byte3 ;
        unsigned char   Byte4 ;
        unsigned char   Byte5 ;
        unsigned char   Byte6 ;
        unsigned char   Byte7 ;        
    };  
}FRAME_Buffer0;

union
{
    struct
    {
        unsigned int    SID ;
        unsigned long   EID ;
        unsigned char   DLC ;
        unsigned char   IDE_BIT ;
        unsigned char   DATA_IN ;
        unsigned char   DATA_IN1 ;
        unsigned int    DataWord0 ;
        unsigned int    DataWord1 ;
        unsigned int    DataWord2 ;
        unsigned int    DataWord3 ;
    };
    struct
    {
        unsigned int    Null1 ;
        unsigned long   Null2 ;
        unsigned char   Null3 ;
        unsigned char   Null4 ;
        unsigned char   Null5 ;
        unsigned char   Null6 ;
        unsigned char   Byte0 ;
        unsigned char   Byte1 ;
        unsigned char   Byte2 ;
        unsigned char   Byte3 ;
        unsigned char   Byte4 ;
        unsigned char   Byte5 ;
        unsigned char   Byte6 ;
        unsigned char   Byte7 ;        
    };  
}FRAME_Buffer1;

char   CDC_Buffer[64];     

// Define ECAN Message Buffers
__eds__ ECAN1MSGBUF ecan1msgBuf __attribute__( (space(eds), aligned(ECAN1_MSG_BUF_LENGTH * 16)) );

// *****************************************************************************
// Section: Static Function declaration
// *****************************************************************************
void                            OscConfig( void );
void                            ClearIntrflags( void );
void                            Ecan1WriteMessage( void );
void                            CAN_Send_CLOCK_Message( void ) ;
void                            CAN_Send_VR1_Message( void ) ;
void                            CAN_Send_REMOTE_Message(void);

void                            LINE_12_Initial(void) ;
void                            LINE_12_Write_Default(void) ;

void                            SEVEN_SEGMENT_SCAN(void) ;
void                           IO_ADC_Config(void) ;

void                           Time_Buf_to_LINE1(void) ;
void                           ADC_Buf_to_LINE2( void) ;
 
void                           UART1_Initial(void) ;
 
void                           BOARD_ID_Get(void) ;
void                           Update_CLOCK1(void) ;
void                           Check_CAN_FIFO( void );
void                           EXT_CLOCK_to_LINE1(void) ;
void                           EXT_ADC_Buf_to_LINE2(void ) ;   
 
 unsigned long            MY_CANID ;

const unsigned long     ID_TABLE[20] = {0x100, 0x110,0x120,0x130,0x140,0x150,0x160,0x170,0x180,0x190,
                                        0x200, 0x210,0x220,0x230,0x240,0x250,0x260,0x270,0x280,0x290}; 
unsigned int                StateMachine ;
unsigned int                Station_CheckIndex = 1 ;
unsigned int                RXFUL_Loop ;
unsigned int                RXFUL_Var ;
unsigned int                Flag_Timer1 = 0 ;
unsigned int                Flag_Timer1_Counter = 0 ;
unsigned int                Flag_Timer1_1S = 0 ;
unsigned int                Flag_Timer1_100MS = 0 ;
unsigned int                Flag_Timer1_CANTX = 0 ;
unsigned int                VR1_Value = 0 ;
unsigned int                EXT_VR1_Value = 0 ;
unsigned int                LED1_Timer= 0 ;
unsigned int                LED2_Timer= 0 ;


unsigned int                CAN_ID, CLOCK_ID, VR1_ID ;
unsigned int                REMOTE_ID ;

extern unsigned char            FIRST_LINE_Data[6];
extern unsigned char            SECOND_LINE_Data[6];
extern const unsigned char    SEVEN_SEG_PATTERN[16];


// *****************************************************************************
// Section: Function definition
// *****************************************************************************

/******************************************************************************
 * Function:        int main(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        Sets CPU clock and initializes CAN1 and CAN2.
 *****************************************************************************/
int main(void)
{

    /* Configure Oscillator Clock Source     */
    //OscConfig();
    SYSTEM_Initialize(SYSTEM_STATE_USB_START);
    USBDeviceInit();
    USBDeviceAttach();
 
    LED_Enable ( LED_D1 ) ;     
    LED_Enable ( LED_D2 ) ; 
    LED_Enable ( LED_D3 ) ; 
 
    LED_Off(LED_D1) ;    
    LED_Off(LED_D2) ;
    LED_Off(LED_D3) ;
    
    IO_ADC_Config();
    /* Clear Interrupt Flags                 */
    ClearIntrflags();
    
    if (!PIN_DSW3)
    {
        MY_CANID =   0x400 ;
        CLOCK_ID = MY_CANID ;
        VR1_ID = MY_CANID + 0x08 ;
        REMOTE_ID = DEF_REMOTE_ID ;        
    }
    else
    {
        MY_CANID =   0x600 ;
        CLOCK_ID = MY_CANID ;
        VR1_ID = MY_CANID + 0x08 ;   
        REMOTE_ID = DEF_REMOTE_ID ;
    }
        LCM_Init( ) ;
        Timer1_Initial( ) ;
            
            LCM_PutROMString( (const unsigned char*)"CAN Monitor  Hex") ;
                LCM_SetCursor( 0,1 ) ;
                LCM_PutROMString( (const unsigned char*)"125K ID=         " ) ;
        
                if (MY_CANID == 0x400)
                {
                    LCM_SetCursor( 8,1 ) ;   
                    LCM_PutROMString( (const unsigned char*)"400h" ) ;                 
                }
                else 
                {
                    LCM_SetCursor( 8,1 ) ;   
                    LCM_PutROMString( (const unsigned char*)"600h" ) ;                                
                }
                
    Ecan1Init();
    DMA0Init();
    DMA2Init();
    
    UART1_Initial( );

 /* Enable ECAN1 TX and RX Interrupt */
    IEC2bits.C1IE = 1;
    C1INTEbits.TBIE = 1;
    C1INTEbits.RBIE = 1;          
    
    FRAME_Buffer0.DATA_IN = 0 ;
    FRAME_Buffer1.DATA_IN = 0 ;   
    
    StateMachine = 0 ;  
    
            CLOCK1.Hour =0 ;
            CLOCK1.Minute = 0 ;
            CLOCK1.Second = 0 ;
            EXT_CLOCK1.Hour =0 ;
            EXT_CLOCK1.Minute = 0 ;
            EXT_CLOCK1.Second = 0 ;
            CANTX_FLAG.ALL_Busy = 0 ;

        LINE_12_Initial( ); 
        LINE_12_Write_Default () ;   
            
    while( 1 )
    {  
        Check_CAN_FIFO();
        
     if ( Flag_Timer1 ==1)                       // 2 ms interval 
        {
            Flag_Timer1 = 0 ;
            SEVEN_SEGMENT_SCAN( ) ;         // 7 Segment Display scan routine 
        }
            
        if (Flag_Timer1_100MS && USBGetDeviceState() >= CONFIGURED_STATE)
        {
            LED_Toggle(LED_D3) ;
            Flag_Timer1_100MS = 0 ;
        }
        
    if( USBGetDeviceState() >= CONFIGURED_STATE )
    {
        CDCTxService();
        if ( (FRAME_Buffer1.DATA_IN) && (USBUSARTIsTxTrfReady() == true))
        {
                    
                    if ( !PIN_DSW6)
                    sprintf(CDC_Buffer,"R: Hex ID=%3x DLen=%1d Data = %2x, %2x, %2x, %2x, %2x, %2x, %2x, %2x\r\n", 
                            FRAME_Buffer1.SID,FRAME_Buffer1.DLC, FRAME_Buffer1.Byte0,FRAME_Buffer1.Byte1,FRAME_Buffer1.Byte2,FRAME_Buffer1.Byte3, 
                            FRAME_Buffer1.Byte4,FRAME_Buffer1.Byte5,FRAME_Buffer1.Byte6,FRAME_Buffer1.Byte7);
                    else
                    sprintf(CDC_Buffer,"R: Dec ID=%4d  DLen=%1d  Data = %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d\r\n", 
                            FRAME_Buffer1.SID,FRAME_Buffer1.DLC, FRAME_Buffer1.Byte0,FRAME_Buffer1.Byte1,FRAME_Buffer1.Byte2,FRAME_Buffer1.Byte3, 
                            FRAME_Buffer1.Byte4,FRAME_Buffer1.Byte5,FRAME_Buffer1.Byte6,FRAME_Buffer1.Byte7);                        
                    
                    putsUSBUSART(CDC_Buffer) ; 
                    LED_On(LED_D1) ;
                    LED1_Timer = LED1_ON_TIME ;
                    CDCTxService();

            Nop() ;
            Nop() ;
            Nop() ;
            FRAME_Buffer1.DATA_IN = 0 ;
            
        }
    }
        

        if (Flag_Timer1_CANTX == 1 )            // 100ms interval
        {
             AD1CON1bits.SAMP = 0 ;             // Start ADC Conversion 
                while ( !AD1CON1bits.DONE) ;
                VR1_Value = ADC1BUF0 ;  
                    ADC_Buf_to_LINE2( ) ;    
                          if (!SW_BT2)
                          {
                              if ((USBUSARTIsTxTrfReady() == true) && (USBGetDeviceState() >= CONFIGURED_STATE ))
                              {
                                CAN_Send_VR1_Message( ) ;
                                if ( !PIN_DSW6)
                                    sprintf(CDC_Buffer,"T: Hex ID=%3x DLen=4 Data = %2x, %2x,  00,  00\r\n", 
                                            VR1_ID,(unsigned char)VR1_Value & 0x00ff,((unsigned char)(VR1_Value >> 8)) & 0x00ff);
                                else
                                    sprintf(CDC_Buffer,"T: Dec ID=%4d DLen=4 Data = %3d, %3d,  00,  00\r\n", 
                                            VR1_ID,(unsigned char)VR1_Value & 0x00ff,((unsigned char)(VR1_Value >> 8)) & 0x00ff);                                    
                                
                                putsUSBUSART(CDC_Buffer) ;
                                LED_On(LED_D2) ;
                                LED2_Timer = LED2_ON_TIME ;
                                CDCTxService();
                              }
                          }
                            if (!SW_BT4)
                            {     
                              if ((USBUSARTIsTxTrfReady() == true) && (USBGetDeviceState() >= CONFIGURED_STATE ))
                              {
                                CAN_Send_CLOCK_Message( ) ;
                                if ( !PIN_DSW6)
                                    sprintf(CDC_Buffer,"T: Hex ID=%3x DLen=8 Data = %2x,  00, %2x,  00, %2x,  00,  00,  00\r\n", 
                                            CLOCK_ID,(unsigned char)CLOCK1.Second,(unsigned char)CLOCK1.Minute,(unsigned char)CLOCK1.Hour );    
                                else
                                    sprintf(CDC_Buffer,"T: Dec ID=%4d DLen=8 Data = %3d,  00, %3d,  00, %3d,  00,  00,  00\r\n", 
                                            CLOCK_ID,(unsigned char)CLOCK1.Second,(unsigned char)CLOCK1.Minute,(unsigned char)CLOCK1.Hour );                                    
                                
                                putsUSBUSART(CDC_Buffer) ;
                                LED_On(LED_D2) ;
                                LED2_Timer = LED2_ON_TIME ;                                
                                CDCTxService();
                              }
                            }
                              
                             if (!SW_BT3)
                            {     
                              if ((USBUSARTIsTxTrfReady() == true) && (USBGetDeviceState() >= CONFIGURED_STATE ))
                              {
                                CAN_Send_REMOTE_Message( ) ;
                                if ( !PIN_DSW6)
                                    sprintf(CDC_Buffer,"T - RTR : Hex ID=%3x DLen=0\r\n", REMOTE_ID);    
                                else
                                    sprintf(CDC_Buffer,"T - RTR : Dec ID=%4d DLen=0\r\n", REMOTE_ID);                                    
                                
                                putsUSBUSART(CDC_Buffer) ;
                                LED_On(LED_D2) ;
                                LED2_Timer = LED2_ON_TIME ;                                
                                CDCTxService();
                              }                             
                            }                                
                           
                          
                            Flag_Timer1_CANTX = 0 ;
        }
        
        if (Flag_Timer1_1S == 1 )               //  1 Second interval
        {               
            Update_CLOCK1( ) ;        
            Flag_Timer1_1S = 0 ;
            Time_Buf_to_LINE1( ) ;
            
            if (!PIN_DSW6)
            {
               LCM_SetCursor( 13,0 ) ;
               LCM_PutROMString( (const unsigned char*)"Hex") ;
            }
            else
            {
               LCM_SetCursor( 13,0 ) ;
               LCM_PutROMString( (const unsigned char*)"Dec") ;                
            }
            
            if (!PIN_DSW1)
            {
               LCM_SetCursor( 14,1 ) ;
               LCM_PutROMString( (const unsigned char*)"HS") ;
            }
            else
            {
               LCM_SetCursor( 14,1 ) ;
               LCM_PutROMString( (const unsigned char*)"LS") ;                
            }            
            
//             if ( !PIN_DSW6)
//             CAN_Send_CLOCK_Message( ) ;
             switch ( StateMachine)
            {
                case  0x00 :
                                    // Time_Buf_to_LINE1( ) ;    
                                    
                                    break ;               
                case  0x01 :     
                                    break ;  
                case  0x02 :
                                    break ;
                default :
                                    break ;
            }
        }
       
        Nop() ;
    }
}

void    Check_CAN_FIFO( void )
{
         if ( C1FIFObits.FBP != C1FIFObits.FNRB )
        {   
             RXFUL_Var = C1FIFObits.FNRB ;
            
            FRAME_Buffer1.SID = (ecan1msgBuf[RXFUL_Var][0] >> 2 ) & 0x07ff ;
            FRAME_Buffer1.DLC = ecan1msgBuf[RXFUL_Var][2] & 0x000f ;
            {
                FRAME_Buffer1.DataWord0 = ecan1msgBuf[RXFUL_Var][3] ;           //  Read ADC Value -- It at 1st word
                FRAME_Buffer1.DataWord1 = ecan1msgBuf[RXFUL_Var][4] ;
                FRAME_Buffer1.DataWord2 = ecan1msgBuf[RXFUL_Var][5] ;           
                FRAME_Buffer1.DataWord3 = ecan1msgBuf[RXFUL_Var][6] ;
                FRAME_Buffer1.DATA_IN = 1 ; 
                Nop() ;
                Nop() ;
            }
            
            RXFUL_Var = 0xfffe ;
            for ( RXFUL_Loop = 0 ; RXFUL_Loop < C1FIFObits.FNRB; RXFUL_Loop++)
                RXFUL_Var <<= 1 ;
                C1RXFUL1 &= RXFUL_Var ;         // Clear RXFUL Flag ...
     
        }     
    
}

void    Update_CLOCK1( void )
{
    CLOCK1.Second ++ ;
    if ( CLOCK1.Second >= 60)
    {
        CLOCK1.Second = 0 ;
        CLOCK1.Minute ++ ;
        if (CLOCK1.Minute >= 60)
        {
            CLOCK1.Minute = 0 ;
            CLOCK1.Hour ++ ;
            if ( CLOCK1.Hour >= 12)
                CLOCK1.Hour = 0;
        }
    }
}

// Configure UART1 to work as 19200 bps 
//  U1TX = RP99
//  U1RX = RP98
void    UART1_Initial(void)             
{
    U1MODE = 0x2800 ;
    U1STA = 0x0000 ;
    U1BRG = 194 ;
    
        RPINR18bits.U1RXR = 98 ;         // set U1RX to RP98  (52)
        RPOR8bits.RP99R = 1 ;               // RPOR12bits.RP109R = 1 ; set U1TX to RP99        (51)       Function Code for U1TX is 000001       
        
    U1STAbits.UTXEN = 1 ;
    U1MODEbits.UARTEN = 1 ;
    IEC0bits.U1RXIE = 1 ;

}

void    LINE_12_Write_Default(void)
{
    FIRST_LINE_Data[0] =  SEVEN_SEG_PATTERN[10] ;
       FIRST_LINE_Data[1] =  SEVEN_SEG_PATTERN[10] ;
           FIRST_LINE_Data[2] =  SEVEN_SEG_PATTERN[10] ;
               FIRST_LINE_Data[3] =  SEVEN_SEG_PATTERN[10] ;
                   FIRST_LINE_Data[4] =  SEVEN_SEG_PATTERN[10] ;
                       FIRST_LINE_Data[5] =  SEVEN_SEG_PATTERN[10] ;
                       
    SECOND_LINE_Data[0] =  SEVEN_SEG_PATTERN[10] ;
       SECOND_LINE_Data[1] =  SEVEN_SEG_PATTERN[10] ;
            SECOND_LINE_Data[2] =  SEVEN_SEG_PATTERN[10] ;
               SECOND_LINE_Data[3] =  SEVEN_SEG_PATTERN[10] ;
                    SECOND_LINE_Data[4] =  SEVEN_SEG_PATTERN[10] ;
                       SECOND_LINE_Data[5] =  SEVEN_SEG_PATTERN[10] ;    
}

void    Time_Buf_to_LINE1(void)
{
    FIRST_LINE_Data[0] =  SEVEN_SEG_PATTERN[CLOCK1.Hour / 10 ] ;
       FIRST_LINE_Data[1] =  SEVEN_SEG_PATTERN[CLOCK1.Hour %10] ;
           FIRST_LINE_Data[2] =  SEVEN_SEG_PATTERN[CLOCK1.Minute / 10] ;
               FIRST_LINE_Data[3] =  SEVEN_SEG_PATTERN[CLOCK1.Minute %10] ;
                   FIRST_LINE_Data[4] =  SEVEN_SEG_PATTERN[CLOCK1.Second /10] ;
                       FIRST_LINE_Data[5] =  SEVEN_SEG_PATTERN[CLOCK1.Second % 10] ;       
}

void    EXT_CLOCK_to_LINE1(void)
{
    FIRST_LINE_Data[0] =  SEVEN_SEG_PATTERN[EXT_CLOCK1.Hour / 10 ] ;
       FIRST_LINE_Data[1] =  SEVEN_SEG_PATTERN[EXT_CLOCK1.Hour %10] ;
           FIRST_LINE_Data[2] =  SEVEN_SEG_PATTERN[EXT_CLOCK1.Minute / 10] ;
               FIRST_LINE_Data[3] =  SEVEN_SEG_PATTERN[EXT_CLOCK1.Minute %10] ;
                   FIRST_LINE_Data[4] =  SEVEN_SEG_PATTERN[EXT_CLOCK1.Second /10] ;
                       FIRST_LINE_Data[5] =  SEVEN_SEG_PATTERN[EXT_CLOCK1.Second % 10] ;       
}
 
void    ADC_Buf_to_LINE2(void)
{
    unsigned int    Temp_Value ;
    
    Temp_Value = VR1_Value  ;
     SECOND_LINE_Data[0] =  SEVEN_SEG_PATTERN[Temp_Value/1000 ] ;
     Temp_Value = Temp_Value - ( 1000 * (Temp_Value/1000)) ;
       SECOND_LINE_Data[1] =  SEVEN_SEG_PATTERN[Temp_Value/100] ;
       Temp_Value = Temp_Value - (100 * (Temp_Value/100));       
           SECOND_LINE_Data[2] =  SEVEN_SEG_PATTERN[Temp_Value/ 10] ;
               SECOND_LINE_Data[3] =  SEVEN_SEG_PATTERN[Temp_Value%10] ;
                   SECOND_LINE_Data[4] =  SEVEN_SEG_PATTERN[10] ;
                       SECOND_LINE_Data[5] =  SEVEN_SEG_PATTERN[10] ;       
}

void    EXT_ADC_Buf_to_LINE2(void)
{
    unsigned int    Temp_Value ;
    
    Temp_Value = EXT_VR1_Value  ;
     SECOND_LINE_Data[0] =  SEVEN_SEG_PATTERN[Temp_Value/1000 ] ;
     Temp_Value = Temp_Value - ( 1000 * (Temp_Value/1000)) ;
       SECOND_LINE_Data[1] =  SEVEN_SEG_PATTERN[Temp_Value/100] ;
       Temp_Value = Temp_Value - (100 * (Temp_Value/100));       
           SECOND_LINE_Data[2] =  SEVEN_SEG_PATTERN[Temp_Value/ 10] ;
               SECOND_LINE_Data[3] =  SEVEN_SEG_PATTERN[Temp_Value%10] ;
                   SECOND_LINE_Data[4] =  SEVEN_SEG_PATTERN[10] ;
                       SECOND_LINE_Data[5] =  SEVEN_SEG_PATTERN[10] ;       
}

void    BOARD_ID_Get(void)
{
    if (PIN_DSW1)   BOARD_ID.DSW1 = 0 ;
        else                BOARD_ID.DSW1 = 1 ;                    
    if (PIN_DSW2)   BOARD_ID.DSW2 = 0 ;
    else                    BOARD_ID.DSW2  = 1 ;
    
    if (PIN_DSW3)   BOARD_ID.DSW3 = 0 ;
        else                BOARD_ID.DSW3 = 1 ;                    
    if (PIN_DSW4)   BOARD_ID.DSW4 = 0 ;
    else                    BOARD_ID.DSW4  = 1 ;
    
    if (PIN_DSW5)   BOARD_ID.DSW5 = 0 ;
        else                BOARD_ID.DSW5 = 1 ;                    
    if (PIN_DSW6)   BOARD_ID.DSW6 = 0 ;
    else                    BOARD_ID.DSW6  = 1 ;  

}


//
//
//void    Delay_for_Time(void)
//{
//    volatile unsigned long    Loop1 ;
//    volatile unsigned long    Loop2 ;
//    
//    for (Loop2 = 0 ; Loop2 < 10; Loop2 ++)
//        for (Loop1 = 0 ; Loop1 < 100 ; Loop1 ++ ) ;
//}

  void   IO_ADC_Config(void)
  {
      ANSELA = 0x0000 ;
      ANSELB = 0x0000 ;
      ANSELC = 0x0000 ;
      ANSELE = 0x0000 ;
      ANSELD = 0x0000 ;
      ANSELG = 0x0000 ; 
      
      DIR_LED1 = 0 ;
      DIR_LED2 = 0 ;
      DIR_LED3 = 0 ;
      DIR_LED4 = 0 ;
      
      AD1CON1 = 0b0001010000000110  ;                    // Clearing sample bit (SAMP) ends sampling and start conversion, ASAM = 1 
      AD1CON2 = 0b0000000000000000 ;
      AD1CON3 = 0b1000001000000010 ;
      AD1CON4 = 0b0000000000000000 ;
      AD1CHS123 = 0x00 ;
      AD1CHS0 = 0x00 ;                                              // Select AN0 as Positive INPUT of ADC
      ANSELBbits.ANSB0 = 1 ;                                   // Set RB0 as Analog input !
      AD1CON1bits.ADON = 1 ;
           
  }

void    CAN_Send_CLOCK_Message(void)
{
    if ( ! C1TR01CONbits.TXREQ0   )
    {
                Ecan1WriteTxMsgBufId( 0, CLOCK_ID, 0, 0 );
                Ecan1WriteTxMsgBufData( 0, 8,  (unsigned int) CLOCK1.Second,
                                                            (unsigned int) CLOCK1.Minute, 
                                                            (unsigned int) CLOCK1.Hour , 
                                                            0x00);
                C1TR01CONbits.TXREQ0 = 1 ; 
                 CANTX_FLAG.TX0_Busy = 1 ;
    }
}

void    CAN_Send_VR1_Message(void)
{
    if ( ! C1TR01CONbits.TXREQ1  )
    {
                Ecan1WriteTxMsgBufId( 1, VR1_ID, 0, 0 );
                Ecan1WriteTxMsgBufData(    1, 4 ,   VR1_Value ,
                                                        (unsigned int) BOARD_ID.ID_Byte,
                                                        0x00,
                                                        0x00 );
                C1TR01CONbits.TXREQ1 = 1 ; 
                CANTX_FLAG.TX1_Busy = 1 ;
    }
}

void    CAN_Send_REMOTE_Message(void)
{
    if ( ! C1TR23CONbits.TXREQ2 )
    {
                Ecan1WriteTxMsgBufId( 2, REMOTE_ID, 0, 1 );                     // Last parameter == 1 for REMOTE Request
                Ecan1WriteTxMsgBufData(    2, 0 , 0x00, 0x00, 0x00 ,0x00 );
                C1TR23CONbits.TXREQ2 = 1 ; 
                CANTX_FLAG.TX2_Busy = 1 ;
    }
}



/******************************************************************************
 * Function:        void ClearIntrflags(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:         Clears all the interrupt flag registers.
 *****************************************************************************/
void ClearIntrflags( void )
{
    /* Clear Interrupt Flags */
    IFS0 = 0;
    IFS1 = 0;
    IFS2 = 0;
    IFS3 = 0;
    IFS4 = 0;
}

/******************************************************************************
 * Function:        void OscConfig(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:       This function configures the Oscillator to work at 60MHz.
 *****************************************************************************/
void OscConfig( void )
{
    /*  Configure Oscillator to operate the device at 60Mhz
     Fosc= Fin*M/(N1*N2), Fcy=Fosc/2
     Fosc= 8M*60/(2*2)=120Mhz for 8M input clock */
    PLLFBD = 58;                                /* M=60 */
    CLKDIVbits.PLLPOST = 0;                     /* N1=2 */
    CLKDIVbits.PLLPRE = 0;                      /* N2=2 */
    OSCTUN = 0;                                 /* Tune FRC oscillator, if FRC is used */

    /* Disable Watch Dog Timer */
    RCONbits.SWDTEN = 0;

    /* Clock switch to incorporate PLL*/
    __builtin_write_OSCCONH( 0x03 );            // Initiate Clock Switch to Primary

    // Oscillator with PLL (NOSC=0b011)
    __builtin_write_OSCCONL( OSCCON || 0x01 );  // Start clock switching
    while( OSCCONbits.COSC != 0b011 );

    // Wait for Clock switch to occur
    /* Wait for PLL to lock */
    while( OSCCONbits.LOCK != 1 )
    { };
}

/******************************************************************************
 * Function:      void __attribute__((interrupt, no_auto_psv))_C1Interrupt(void)
 *
 * PreCondition:  None
 *
 * Input:         None
 *
 * Output:        None
 *
 * Side Effects:  None
 *
 * Overview:      Interrupt service routine to handle CAN1 Transmit and
 *                recieve interrupt.
 *****************************************************************************/
void __attribute__ ( (interrupt, no_auto_psv) ) _C1Interrupt( void )
{
    IFS2bits.C1IF = 0;      // clear interrupt flag
    if( C1INTFbits.TBIF )
    {
        if (CANTX_FLAG.TX0_Busy)
        {   CANTX_FLAG.TX0_Busy = 0 ;   }
        if (CANTX_FLAG.TX1_Busy)
        {   CANTX_FLAG.TX1_Busy = 0 ;   }
        if (CANTX_FLAG.TX2_Busy)
        {   CANTX_FLAG.TX2_Busy = 0 ;   }
        
        C1INTFbits.TBIF = 0;
    }

    if( C1INTFbits.RBIF )
    {
        C1INTFbits.RBIF = 0;
        if ( C1RXFUL1bits.RXFUL4 ) 
        {
            FRAME_Buffer0.SID = (ecan1msgBuf[4][0]>> 2) & 0x07ff ;
            FRAME_Buffer0.DLC = ecan1msgBuf[4][2] & 0x001f ;
            FRAME_Buffer0.DataWord0 = ecan1msgBuf[4][3] ;
            FRAME_Buffer0.DataWord1 = ecan1msgBuf[4][4] ;
            FRAME_Buffer0.DataWord2 = ecan1msgBuf[4][5] ;
            FRAME_Buffer0.DataWord3 = ecan1msgBuf[4][6] ;
            FRAME_Buffer0.DATA_IN = 1 ;                             // Set DATA_IN to indicate data valid !!
            C1RXFUL1bits.RXFUL4 = 0 ;
        }
        Nop();
        Nop();
        Nop();
    }
}



//------------------------------------------------------------------------------
//    DMA interrupt handlers
//------------------------------------------------------------------------------
/******************************************************************************
 * Function:   void __attribute__((interrupt, no_auto_psv)) _DMA0Interrupt(void)
 *
 * PreCondition:  None
 *
 * Input:         None
 *
 * Output:        None
 *
 * Side Effects:  None
 *
 * Overview:      Interrupt service routine to handle DMA0interrupt
 *****************************************************************************/
void __attribute__ ( (interrupt, no_auto_psv) ) _DMA0Interrupt( void )
{
    IFS0bits.DMA0IF = 0;    // Clear the DMA0 Interrupt Flag;
}

/******************************************************************************
 * Function:   void __attribute__((interrupt, no_auto_psv)) _DMA1Interrupt(void)
 *
 * PreCondition:  None
 *
 * Input:         None
 *
 * Output:        None
 *
 * Side Effects:  None
 *
 * Overview:      Interrupt service routine to handle DMA1interrupt
 *****************************************************************************/
void __attribute__ ( (interrupt, no_auto_psv) ) _DMA2Interrupt( void )
{
    IFS1bits.DMA2IF = 0;    // Clear the DMA1 Interrupt Flag;
}
/*******************************************************************************
 End of File
*/
